@@ -8,4 +8,4 @@ import ankipandas.collection
 from ankipandas.collection import Collection
 from ankipandas.paths import find_db, db_path_input
 from ankipandas.ankidf import AnkiDataFrame
-from ankipandas.util.log import log, set_log_level
+from ankipandas.util.log import log, set_log_level, set_debug_log_level
