@@ -8,7 +8,7 @@ import time
 import numpy as np
 import pandas as pd
 import pathlib
-from typing import Union, List, Dict, Optional
+from typing import Union, List, Dict, Optional, Iterable
 
 # ours
 import ankipandas.paths
@@ -20,20 +20,26 @@ from ankipandas.util.misc import invert_dict, flatten_list_list
 from ankipandas.util.log import log
 from ankipandas.util.checksum import field_checksum
 from ankipandas.util.guid import guid as generate_guid
-from ankipandas.util.types import *
+from ankipandas.util.types import (
+    is_list_list_like,
+    is_list_like,
+    is_list_dict_like,
+    is_dict_list_like,
+)
 
 
 class AnkiDataFrame(pd.DataFrame):
     #: Additional attributes of a :class:`AnkiDataFrame` that a normal
-    #: :class:`pandas.DataFrame` does not posess. These will be copied in the
+    #: :class:`pandas.DataFrame` does not possess. These will be copied in the
     #: constructor.
-    _attributes = (
+    #: See https://pandas.pydata.org/pandas-docs/stable/development/extending.html
+    _metadata = [
         "col",
         "_anki_table",
         "fields_as_columns_prefix",
         "_fields_format",
         "_df_format",
-    )
+    ]
 
     def __init__(self, *args, **kwargs):
         """ Initializes a blank :class:`AnkiDataFrame`.
@@ -59,6 +65,8 @@ class AnkiDataFrame(pd.DataFrame):
         # todo: document
         self.col = None
 
+        # noinspection PyTypeChecker
+        # gets set by _get_table
         #: Type of anki table: 'notes', 'cards' or 'revlog'. This corresponds to
         #: the meaning of the ID row.
         self._anki_table = None  # type: str
@@ -70,38 +78,17 @@ class AnkiDataFrame(pd.DataFrame):
         #:   or ``anki`` (default)
         self._fields_format = "anki"
 
+        # gets set by _get_table
+        # noinspection PyTypeChecker
         #: Overal structure of the dataframe ``anki``, ``ours``, ``in_progress``
         self._df_format = None  # type: str
 
-        # todo: is this serving any purpose? Coverage shows it never runs.
-        if len(args) == 1 and isinstance(args[0], AnkiDataFrame):
-            self._copy_attrs_from(args[0])
-
     @property
     def _constructor(self):
         """ This needs to be overriden so that any DataFrame operations do not
         return a :class:`pandas.DataFrame` but a :class:`AnkiDataFrame`."""
 
-        def __constructor(*args, **kw):
-            df = self.__class__(*args, **kw)
-            self._copy_attrs_to(df)
-            return df
-
-        return __constructor
-
-    def _copy_attrs_to(self, df):
-        """ Copy all additional attributes of this class to another instance.
-        Also see :attr:`self._attributes`.
-        """
-        for attr in self._attributes:
-            df.__dict__[attr] = getattr(self, attr, None)
-
-    def _copy_attrs_from(self, df):
-        """ Copy all additional attributes of this class from another instance.
-        Also see :attr:`self._attributes`.
-        """
-        for attr in self._attributes:
-            self.__dict__[attr] = getattr(df, attr, None)
+        return AnkiDataFrame
 
     # Constructors
     # ==========================================================================
@@ -133,18 +120,63 @@ class AnkiDataFrame(pd.DataFrame):
 
     def append(self, *args, **kwargs):
         ret = pd.DataFrame.append(self, *args, **kwargs)
-        self._copy_attrs_to(ret)
         ret.astype(_columns.dtype_casts2[self._anki_table])
         return ret
 
-    def update(self, *args, **kwargs):
-        super(AnkiDataFrame, self).update(*args, **kwargs)
+    def update(self, other, force=False, **kwargs):
+        if not force and isinstance(other, AnkiDataFrame):
+            if other._df_format != self._df_format:
+                raise ValueError(
+                    "You're trying to update an AnkiDataFrame in format {f1}"
+                    " using another AnkiDataFrame in format {f2}. That doesn't "
+                    "sound like a good idea. However you can still do this "
+                    "using the force=True option.".format(
+                        f1=self._df_format, f2=other._df_format
+                    )
+                )
+            if other._anki_table != self._anki_table:
+                raise ValueError(
+                    "You're trying to update an AnkiDataFrame of table {f1} "
+                    "with an AnkiDataFrame of table {f2}. That doesn't sound"
+                    " like a good idea. However you can still do this using "
+                    "the force=True option.".format(
+                        f1=self._anki_table, f2=other._anki_table
+                    )
+                )
+            if self._anki_table == "notes":
+                if other._fields_format != self._fields_format:
+                    raise ValueError(
+                        "You are trying to update a notes AnkiDataFrame where "
+                        "the fields are in format '{f1}' with a notes "
+                        "AnkiDataFrame where the fields are in format '{f2}'. "
+                        "That doesn't sound like a good idea. However you can "
+                        "still do this using the force=True option. "
+                        "Or you simply ensure that both have the same format"
+                        " using the fields_as_columns() or fields_as_list() "
+                        "method.".format(
+                            f1=self._fields_format, f2=other._fields_format
+                        )
+                    )
+        super(AnkiDataFrame, self).update(other, **kwargs)
         # Fix https://github.com/pandas-dev/pandas/issues/4094
         for col, typ in _columns.dtype_casts2[self._anki_table].items():
             self[col] = self[col].astype(typ)
 
+    # Checks
     # ==========================================================================
 
+    def check_table_integrity(self):
+        duplicates = self.index[self.index.duplicated()].tolist()
+        if duplicates:
+            log.critical(
+                "Duplicated indizes in table {} discovered, so something "
+                "definitely went wrong. Please don't ignore this warning. "
+                "These indizes appear more "
+                "than once: {}".format(
+                    self._anki_table, ", ".join(map(str, duplicates))
+                )
+            )
+
     def _invalid_table(self):
         raise ValueError("Invalid table: {}.".format(self._anki_table))
 
@@ -152,7 +184,7 @@ class AnkiDataFrame(pd.DataFrame):
         if self._df_format == "in_progress":
             raise ValueError(
                 "Previous call to normalize() or raw() did not terminate "
-                "succesfully. This is usually a very bad sign, but you can "
+                "successfully. This is usually a very bad sign, but you can "
                 "try calling them again with the force option: raw(force=True) "
                 "or raw(force=True) and see if that works."
             )
@@ -202,7 +234,6 @@ class AnkiDataFrame(pd.DataFrame):
     @property
     def nid(self):
         """ Note ID as :class:`pandas.Series` of integers. """
-        self._check_our_format()
         if self._anki_table == "notes":
             return self.index
         elif self._anki_table == "cards":
@@ -235,7 +266,6 @@ class AnkiDataFrame(pd.DataFrame):
     @property
     def cid(self):
         """ Card ID as :class:`pandas.Series` of integers. """
-        self._check_our_format()
         if self._anki_table == "cards":
             return self.index
         if self._anki_table == "revs":
@@ -302,7 +332,6 @@ class AnkiDataFrame(pd.DataFrame):
     @property
     def mid(self):
         """ Model ID as :class:`pandas.Series` of integers. """
-        self._check_our_format()
         if self._anki_table in ["notes"]:
             if "nmodel" not in self.columns:
                 raise ValueError(
@@ -332,7 +361,6 @@ class AnkiDataFrame(pd.DataFrame):
     @property
     def did(self):
         """ Deck ID as :class:`pandas.Series` of integers. """
-        self._check_our_format()
         if self._anki_table == "cards":
             if "cdeck" not in self.columns:
                 raise ValueError(
@@ -365,7 +393,6 @@ class AnkiDataFrame(pd.DataFrame):
         """ Original deck ID for cards in filtered deck as
         :class:`pandas.Series` of integers.
         """
-        self._check_our_format()
         if self._anki_table == "cards":
             if "odeck" not in self.columns:
                 raise ValueError(
@@ -378,7 +405,7 @@ class AnkiDataFrame(pd.DataFrame):
                 return self["odeck"].map(raw.get_deck2did(self.db))
         elif self._anki_table == "notes":
             raise ValueError(
-                "The original deck ID (odid) is not availabale for the notes "
+                "The original deck ID (odid) is not available for the notes "
                 "table."
             )
         else:
@@ -426,7 +453,7 @@ class AnkiDataFrame(pd.DataFrame):
             )
         elif self._anki_table == "revs":
             self["nid"] = self.nid
-        return ankipandas.util.dataframe.merge_dfs(
+        ret = ankipandas.util.dataframe.merge_dfs(
             df=self,
             df_add=self.col.notes,
             id_df="nid",
@@ -437,6 +464,7 @@ class AnkiDataFrame(pd.DataFrame):
             columns=columns,
             drop_columns=drop_columns,
         )
+        return ret
 
     def merge_cards(
         self,
@@ -472,7 +500,7 @@ class AnkiDataFrame(pd.DataFrame):
                 "notes table."
             )
         self._check_our_format()
-        return ankipandas.util.dataframe.merge_dfs(
+        ret = ankipandas.util.dataframe.merge_dfs(
             df=self,
             df_add=self.col.cards,
             id_df="cid",
@@ -483,11 +511,12 @@ class AnkiDataFrame(pd.DataFrame):
             prepend=prepend,
             prepend_clash_only=prepend_clash_only,
         )
+        return ret
 
     # Toggle format
     # ==========================================================================
 
-    def fields_as_columns(self, inplace=False):
+    def fields_as_columns(self, inplace=False, force=False):
         """
         In the 'notes' table, the field contents of the notes is contained in
         one column ('flds') by default. With this method, this column can be
@@ -495,26 +524,25 @@ class AnkiDataFrame(pd.DataFrame):
 
         Args:
             inplace: If False, return new dataframe, else update old one
+            force: Internal use
 
         Returns:
             New :class:`pandas.DataFrame` if inplace==True, else None
         """
-        self._check_our_format()
+        if not force:
+            self._check_our_format()
         if not inplace:
             df = self.copy(True)
             df.fields_as_columns(inplace=True)
             return df
 
-        if "nflds" not in self.columns:
-            raise ValueError("Could not find fields column 'nflds'.")
-
         if self._fields_format == "columns":
             log.warning(
                 "Fields are already as columns."
                 " Returning without doing anything."
             )
             return
-        elif self._fields_format == "in_progress":
+        elif self._fields_format == "in_progress" and not force:
             raise ValueError(
                 "It looks like the last call to fields_as_list or"
                 "fields_as_columns was not successful, so you better start "
@@ -527,6 +555,9 @@ class AnkiDataFrame(pd.DataFrame):
                 "Unknown _fields_format: {}".format(self._fields_format)
             )
 
+        if "nflds" not in self.columns:
+            raise ValueError("Could not find fields column 'nflds'.")
+
         self._fields_format = "in_progress"
         # fixme: What if one field column is one that is already in use?
         prefix = self.fields_as_columns_prefix
@@ -549,21 +580,23 @@ class AnkiDataFrame(pd.DataFrame):
         self.drop("nflds", axis=1, inplace=True)
         self._fields_format = "columns"
 
-    def fields_as_list(self, inplace=False):
+    def fields_as_list(self, inplace=False, force=False):
         """
         This reverts :meth:`.fields_as_columns`, all columns that represented
         field contents are now merged into one column 'nflds'.
 
         Args:
             inplace: If False, return new dataframe, else update old one
+            force: Internal use
 
         Returns:
             New :class:`AnkiDataFrame` if inplace==True, else None
         """
-        self._check_our_format()
+        if not force:
+            self._check_our_format()
         if not inplace:
             df = self.copy(True)
-            df.fields_as_list(inplace=True)
+            df.fields_as_list(inplace=True, force=force)
             return df
 
         if self._fields_format == "list":
@@ -571,7 +604,7 @@ class AnkiDataFrame(pd.DataFrame):
                 "Fields are already as list. Returning without doing anything."
             )
             return
-        elif self._fields_format == "in_progress":
+        elif self._fields_format == "in_progress" and not force:
             raise ValueError(
                 "It looks like the last call to fields_as_list or"
                 "fields_as_columns was not successful, so you better start "
@@ -598,7 +631,7 @@ class AnkiDataFrame(pd.DataFrame):
             # might still use them
             to_drop.extend(fields)
         self.drop(to_drop, axis=1, inplace=True)
-        self._fields_format = "fields"
+        self._fields_format = "list"
 
     # Quick access
     # ==========================================================================
@@ -648,7 +681,7 @@ class AnkiDataFrame(pd.DataFrame):
             )
         return sorted(list(self["nmodel"].unique()))
 
-    def has_tag(self, tags=None):
+    def has_tag(self, tags: Optional[Union[Iterable[str], str]] = None):
         """ Checks whether row has a certain tag ('ntags' column).
 
         Args:
@@ -689,7 +722,7 @@ class AnkiDataFrame(pd.DataFrame):
         else:
             return self["ntags"].apply(bool)
 
-    def has_tags(self, tags=None):
+    def has_tags(self, tags: Optional[Union[Iterable[str], str]] = None):
         """ Checks whether row contains at least the supplied tags.
 
         Args:
@@ -718,7 +751,7 @@ class AnkiDataFrame(pd.DataFrame):
         _has_tags = set(tags).issubset
         return self["ntags"].apply(_has_tags)
 
-    def add_tag(self, tags, inplace=False):
+    def add_tag(self, tags: Union[Iterable[str], str], inplace=False):
         """ Adds tag ('ntags' column).
 
         Args:
@@ -746,7 +779,7 @@ class AnkiDataFrame(pd.DataFrame):
 
         self["ntags"] = self["ntags"].apply(_add_tags)
 
-    def remove_tag(self, tags, inplace=False):
+    def remove_tag(self, tags: Union[Iterable[str], str, None], inplace=False):
         """ Removes tag ('ntags' column).
 
         Args:
@@ -779,8 +812,11 @@ class AnkiDataFrame(pd.DataFrame):
     # Compare
     # ==========================================================================
 
-    def was_modified(self, other: pd.DataFrame = None, na=True, _force=False):
+    def was_modified(
+        self, other: Optional[pd.DataFrame] = None, na=True, _force=False
+    ):
         """ Compare with original table, show which rows have changed.
+        Will only compare columns existing in both dataframes.
 
         Args:
             other: Compare with this :class:`pandas.DataFrame`.
@@ -796,23 +832,33 @@ class AnkiDataFrame(pd.DataFrame):
             self._check_our_format()
 
         if other is None:
-            other = self.col._get_original_item(self._anki_table)
+            other = self.col._get_original_item(
+                self._anki_table
+            )  # type: AnkiDataFrame
+
+        self_sf = self
+        if self._fields_format == "columns":
+            self_sf = self.fields_as_list(inplace=False, force=_force)
+
+        cols = sorted(
+            list(set(self_sf.columns).intersection(set(other.columns)))
+        )
 
         other_nids = set(other.index)
-        inters = set(self.index).intersection(other_nids)
-        result = pd.Series(na, index=self.index)
+        inters = set(self_sf.index).intersection(other_nids)
+        result = pd.Series(na, index=self_sf.index)
         new_bools = np.any(
-            other.loc[other.index.isin(inters), list(self.columns)].values
-            != self[self.index.isin(inters)].values,
+            other.loc[other.index.isin(inters), cols].values
+            != self_sf.loc[self_sf.index.isin(inters), cols].values,
             axis=1,
         )
-        result.loc[self.index.isin(inters)] = pd.Series(
-            new_bools, index=result[self.index.isin(inters)].index
+        result.loc[self_sf.index.isin(inters)] = pd.Series(
+            new_bools, index=result[self_sf.index.isin(inters)].index
         )
         return result
 
     def modified_columns(
-        self, other: pd.DataFrame = None, _force=False, only=True
+        self, other: Optional[pd.DataFrame] = None, _force=False, only=True
     ):
         """ Compare with original table, show which columns in which rows
         were modified.
@@ -844,7 +890,7 @@ class AnkiDataFrame(pd.DataFrame):
             columns=cols,
         )
 
-    def was_added(self, other: pd.DataFrame = None, _force=False):
+    def was_added(self, other: Optional[pd.DataFrame] = None, _force=False):
         """ Compare with original table, show which rows were added.
 
         Args:
@@ -868,7 +914,9 @@ class AnkiDataFrame(pd.DataFrame):
         new_indices = set(self.index) - other_ids
         return self.index.isin(new_indices)
 
-    def was_deleted(self, other: pd.DataFrame = None, _force=False):
+    def was_deleted(
+        self, other: Optional[pd.DataFrame] = None, _force=False
+    ) -> List:
         """ Compare with original table, return deleted indizes.
 
         Args:
@@ -932,6 +980,7 @@ class AnkiDataFrame(pd.DataFrame):
         Returns:
             New :class:`AnkiDataFrame` if inplace==True, else None
         """
+
         if not inplace:
             df = self.copy(True)
             df.normalize(inplace=True, force=force)
@@ -976,7 +1025,16 @@ class AnkiDataFrame(pd.DataFrame):
         # IDs
         # ---
 
-        self.set_index(_columns.table2index[table], inplace=True)
+        id_field = _columns.table2index[table]
+        duplicate_ids = self[id_field][self[id_field].duplicated()].tolist()
+        if duplicate_ids:
+            log.critical(
+                "The following IDs occur "
+                "more than once: {}. Please do not use this dataframe.".format(
+                    ", ".join(map(str, duplicate_ids))
+                )
+            )
+        self.set_index(id_field, inplace=True)
 
         if table == "cards":
             self["cdeck"] = self["did"].map(raw.get_did2deck(self.db))
@@ -1007,6 +1065,8 @@ class AnkiDataFrame(pd.DataFrame):
         drop_columns = set(self.columns) - set(_columns.our_columns[table])
         self.drop(drop_columns, axis=1, inplace=True)
 
+        self.check_table_integrity()
+
         self._df_format = "ours"
 
     def raw(self, inplace=False, force=False):
@@ -1069,7 +1129,7 @@ class AnkiDataFrame(pd.DataFrame):
 
         if table == "notes":
             if not self._fields_format == "list":
-                self.fields_as_columns(inplace=True)
+                self.fields_as_list(inplace=True, force=True)
             # Check if success
             if not self._fields_format == "list":
                 raise ValueError(
@@ -1143,6 +1203,8 @@ class AnkiDataFrame(pd.DataFrame):
         for col in new.columns:
             self[col] = new[col]
 
+        self.check_table_integrity()
+
         self._df_format = "anki"
 
     # Write
@@ -1163,7 +1225,7 @@ class AnkiDataFrame(pd.DataFrame):
             "n": len(self),
             "n_modified": sum(self.was_modified(na=False)),
             "n_added": sum(self.was_added()),
-            "n_deleted": sum(self.was_deleted()),
+            "n_deleted": len(self.was_deleted()),
         }
         as_dict["has_changed"] = (
             as_dict["n_modified"] or as_dict["n_added"] or as_dict["n_deleted"]
@@ -1195,6 +1257,7 @@ class AnkiDataFrame(pd.DataFrame):
             idx += 1
         return idx
 
+    # todo: documentation
     def add_card(
         self,
         nid: int,
@@ -1212,6 +1275,28 @@ class AnkiDataFrame(pd.DataFrame):
         cdue: Optional[int] = None,
         inplace=False,
     ):
+        """
+        Similar to :py:meth:`ankipandas.ankipdf.AnkiDataFrame.add_cards`
+
+        Args:
+            nid:
+            cdeck:
+            cord:
+            cmod:
+            cusn:
+            cqueue:
+            ctype:
+            civl:
+            cfactor:
+            creps:
+            clapses:
+            cleft:
+            cdue:
+            inplace:
+
+        Returns:
+
+        """
         return self.add_cards(
             nid=[nid],
             cdeck=cdeck,
@@ -1330,9 +1415,8 @@ class AnkiDataFrame(pd.DataFrame):
             )
         mid = mids[0]
 
-        minfo = raw.get_model_info(self.db)
-
-        available_ords = list(map(lambda x: x["ord"], minfo[mid]["tmpls"]))
+        # fixme: should use function from ankipandas.raw
+        available_ords = raw.get_mid2templateords(self.db)[mid]
         if cord is None:
             cord = available_ords
         elif isinstance(cord, int):
@@ -1773,7 +1857,7 @@ class AnkiDataFrame(pd.DataFrame):
 
         .. note::
 
-            For better performance, it is advisable to use :meth:`add_notes`,
+            For better performance it is advisable to use :meth:`add_notes`
             when adding many notes.
 
         Args:
@@ -1905,15 +1989,15 @@ class AnkiDataFrame(pd.DataFrame):
         df = pd.read_csv(help_path)
         if column == "auto":
             column = list(self.columns)
-        if table is not "all":
+        if table != "all":
             if isinstance(table, str):
                 table = [table]
             df = df[df["Table"].isin(table)]
-        if column is not "all":
+        if column != "all":
             if isinstance(column, str):
                 column = [column]
             df = df[df["Column"].isin(column)]
-        if ankicolumn is not "all":
+        if ankicolumn != "all":
             if isinstance(ankicolumn, str):
                 ankicolumn = [ankicolumn]
             df = df[df["AnkiColumn"].isin(ankicolumn)]
