@@ -11,7 +11,7 @@ import datetime
 import pathlib
 from functools import lru_cache
 import shutil
-from typing import Union
+from typing import Union, Optional
 
 # ours
 from ankipandas.util.log import log
@@ -23,12 +23,19 @@ def _find_db(
     maxdepth=6,
     filename="collection.anki2",
     break_on_first=False,
-    user=None,
+    user: Optional[str] = None,
 ):
     """
     Like find_database but only for one search path at a time. Also doesn't
     raise any error, even if the search path doesn't exist.
 
+    Args:
+        search_path:
+        maxdepth: Maximum depth relative to search_path
+        filename:
+        break_on_first: Break on first search result
+        user: Only search for this user
+
     Returns:
         collection.defaultdict({user: [list of results]})
     """
@@ -60,11 +67,12 @@ def _find_db(
             if break_on_first:
                 log.debug("_find_db: Breaking after first hit.")
                 break
-        if maxdepth and root.count(os.sep) >= maxdepth:
-            log.debug(
-                "_find_db: Abort search at '{}'. "
-                "Max depth exceeded.".format(str(root))
-            )
+        depth = len(pathlib.Path(root).relative_to(search_path).parts)
+        if maxdepth and depth >= maxdepth:
+            # log.debug(
+            #     "_find_db: Abort search at '{}'. "
+            #     "Max depth exceeded.".format(str(root))
+            # )
             del dirs[:]
     return found
 
@@ -103,18 +111,21 @@ def find_db(
             "You can speed this up by specifying a search path or "
             "directly entering the path to your database."
         )
-        # todo: Windows paths?
         search_paths = [
             "~/.local/share/Anki2/",
             "~/Documents/Anki2",
-            "~/Anki2/",
+            pathlib.Path(os.getenv("APPDATA", "~") + "/Anki2/"),
+            "~/.local/share/Anki2",
             pathlib.Path.home(),
         ]
+        search_paths = [
+            pathlib.Path(sp).expanduser().resolve() for sp in search_paths
+        ]
     if break_on_first:
         log.warning(
             "The search will stop at the first hit, so please verify that "
-            "the result is correct (for example in case there might be more"
-            "than one Anki installations)"
+            "the result is correct (for example in case there might be more "
+            "than one Anki installation)"
         )
     if isinstance(search_paths, (str, pathlib.PurePath)):
         search_paths = [search_paths]
@@ -160,7 +171,7 @@ def find_db(
     if len(found) >= 2:
         raise ValueError(
             "Found more than one database belonging to user {} at {}".format(
-                user, ", ".join(found)
+                user, ", ".join(map(str, found))
             )
         )
     found = found[0]
