@@ -25,10 +25,12 @@ from typing import Dict, List, Union
 
 # 3rd
 import pandas as pd
+import numpy as np
 
 # ours
 from ankipandas.util.log import log
 from ankipandas._columns import tables_ours2anki, anki_columns
+from ankipandas.util.misc import nested_dict, defaultdict2dict
 
 CACHE_SIZE = 32
 
@@ -98,10 +100,72 @@ def get_empty_table(table: str) -> pd.DataFrame:
     return pd.DataFrame(columns=anki_columns[table])
 
 
+def _interpret_json_val(val):
+    if isinstance(val, str) and len(val) >= 1:
+        try:
+            return json.loads(val)
+        except json.decoder.JSONDecodeError:
+            return val
+            # msg = (
+            #     "AN ERROR OCCURRED WHILE TRYING TO LOAD INFORMATION "
+            #     "FROM THE DATABASE. PLEASE COPY THE WHOLE INFORMATION"
+            #     " BELOW AND ABOVE AND OPEN A BUG REPORT ON GITHUB!\n\n"
+            # )
+            # msg += "value to be parsed: {}".format(repr(val))
+            # log.critical(msg)
+            # raise
+    else:
+        return val
+
+
+def read_info(db: sqlite3.Connection, table_name: str):
+    """ Get a table from the database and return nested dictionary mapping of
+    it.
+
+    Args:
+        db:
+        table_name:
+
+    Returns:
+
+    """
+    version = get_db_version(db)
+    _df = pd.read_sql_query(
+        "SELECT * FROM {table_name} ".format(table_name=table_name), db
+    )
+    if version == 0:
+        assert len(_df) == 1, _df
+        ret = {}
+        for col in _df.columns:
+            ret[col] = _interpret_json_val(_df[col][0])
+    elif version == 1:
+        ret = nested_dict()
+        cols = _df.columns
+        # todo: this is a hack, but oh well:
+        index_cols = 1
+        if len(_df[cols[0]].unique()) != len(_df):
+            index_cols = 2
+        for row in _df.iterrows():
+            row = row[1].to_list()
+            if index_cols == 1:
+                for icol in range(1, len(cols)):
+                    ret[row[0]][cols[icol]] = _interpret_json_val(row[icol])
+            elif index_cols == 2:
+                for icol in range(2, len(cols)):
+                    ret[row[0]][row[1]][cols[icol]] = _interpret_json_val(
+                        row[icol]
+                    )
+            else:
+                raise ValueError
+    else:
+        raise NotImplementedError
+    return defaultdict2dict(ret)
+
+
 @lru_cache(CACHE_SIZE)
 def get_info(db: sqlite3.Connection) -> dict:
     """
-    Get all other information from the databse, e.g. information about models,
+    Get all other information from the database, e.g. information about models,
     decks etc.
 
     Args:
@@ -110,16 +174,33 @@ def get_info(db: sqlite3.Connection) -> dict:
     Returns:
         Nested dictionary.
     """
-    _df = pd.read_sql_query("SELECT * FROM col ", db)
-    assert len(_df) == 1
-    ret = {}
-    for col in _df.columns:
-        val = _df[col][0]
-        if isinstance(val, str):
-            ret[col] = json.loads(val)
-        else:
-            ret[col] = val
-    return ret
+    return read_info(db, "col")
+
+
+@lru_cache(CACHE_SIZE)
+def get_db_version(db: sqlite3.Connection) -> int:
+    """ Get "version" of database structure
+
+    Args:
+        db:
+
+    Returns: 0: all info (also for decks and models) was in the 'col' table
+        1: separate 'notetypes' and 'decks' tables
+
+    """
+    _tables = (
+        db.cursor()
+        .execute(
+            "SELECT name FROM sqlite_master "
+            "WHERE type ='table' AND name NOT LIKE 'sqlite_%';"
+        )
+        .fetchall()
+    )
+    tables = [ret[0] for ret in _tables]
+    if "decks" in tables:
+        return 1
+    else:
+        return 0
 
 
 # Basic Setters
@@ -203,6 +284,49 @@ def set_table(
     df_new.to_sql(tables_ours2anki[table], db, if_exists="replace", index=False)
 
 
+class NumpyJSONEncoder(json.JSONEncoder):
+    """ JSON Encoder that support numpy datatypes by converting them to
+    built in datatypes. """
+
+    def default(self, obj):
+        if isinstance(obj, np.integer):
+            return int(obj)
+        elif isinstance(obj, np.floating):
+            return float(obj)
+        elif isinstance(obj, np.ndarray):
+            return obj.tolist()
+        else:
+            return super(NumpyJSONEncoder, self).default(obj)
+
+
+def set_info(db: sqlite3.Connection, info: dict) -> None:
+    """ Write back extra info to database
+
+    Args:
+        db: Database (:class:`sqlite3.Connection`)
+        info: Output of :func:`get_info`
+
+    Returns:
+        None
+    """
+
+    def encode_value(value):
+        if isinstance(value, (float, np.floating)):
+            return value
+        elif isinstance(value, (int, np.integer)):
+            return value
+        elif isinstance(value, str):
+            return value
+        else:
+            return json.dumps(value, cls=NumpyJSONEncoder)
+
+    info_json_strings = {
+        key: encode_value(value) for key, value in info.items()
+    }
+    df = pd.DataFrame(info_json_strings, index=[0])
+    df.to_sql("col", db, if_exists="replace", index=False)
+
+
 # Trivially derived getters
 # ==============================================================================
 
@@ -234,7 +358,17 @@ def get_deck_info(db: sqlite3.Connection) -> dict:
     Returns:
         Nested dictionary
     """
-    return get_info(db)["decks"]
+    if get_db_version(db) == 0:
+        _dinfo = get_info(db)["decks"]
+    elif get_db_version(db) == 1:
+        _dinfo = read_info(db, "decks")
+    else:
+        raise NotImplementedError
+
+    if not _dinfo:
+        return {}
+    else:
+        return _dinfo
 
 
 @lru_cache(CACHE_SIZE)
@@ -277,7 +411,16 @@ def get_model_info(db: sqlite3.Connection) -> dict:
     Returns:
         Nested dictionary
     """
-    return {int(key): value for key, value in get_info(db)["models"].items()}
+    if get_db_version(db) == 0:
+        _dinfo = get_info(db)["models"]
+    elif get_db_version(db) == 1:
+        _dinfo = read_info(db, "notetypes")
+    else:
+        raise NotImplementedError
+    if not _dinfo:
+        return {}
+    else:
+        return {int(key): value for key, value in _dinfo.items()}
 
 
 @lru_cache(CACHE_SIZE)
@@ -313,9 +456,14 @@ def get_model2mid(db: sqlite3.Connection) -> Dict[str, int]:
 @lru_cache(CACHE_SIZE)
 def get_mid2sortfield(db: sqlite3.Connection) -> Dict[int, int]:
     """ Mapping of model ID to index of sort field. """
-    minfo = get_model_info(db)
-    _mid2sortfield = {mid: minfo[mid]["sortf"] for mid in minfo}
-    return defaultdict(int, _mid2sortfield)
+    if get_db_version(db) == 0:
+        minfo = get_model_info(db)
+        _mid2sortfield = {mid: minfo[mid]["sortf"] for mid in minfo}
+        return defaultdict(int, _mid2sortfield)
+    else:
+        # fixme: Don't know how to retrieve sort field yet
+        minfo = get_model_info(db)
+        return {mid: 0 for mid in minfo}
 
 
 @lru_cache(CACHE_SIZE)
@@ -323,15 +471,46 @@ def get_mid2fields(db: sqlite3.Connection) -> Dict[int, List[str]]:
     """ Get mapping of model ID to field names.
 
     Args:
-        db: Databse (:class:`sqlite3.Connection`)
+        db: Database (:class:`sqlite3.Connection`)
 
     Returns:
         Dictionary mapping of model ID (mid) to list of field names.
     """
-    minfo = get_model_info(db)
-    return {
-        int(mid): [flds["name"] for flds in minfo[mid]["flds"]] for mid in minfo
-    }
+    if get_db_version(db) == 0:
+        minfo = get_model_info(db)
+        return {
+            int(mid): [flds["name"] for flds in minfo[mid]["flds"]]
+            for mid in minfo
+        }
+    elif get_db_version(db) == 1:
+        finfo = read_info(db, "fields")
+        mid2fields = {
+            int(mid): [finfo[mid][ord]["name"] for ord in finfo[mid]]
+            for mid in finfo
+        }
+        return mid2fields
+    else:
+        raise NotImplementedError
+
+
+@lru_cache(CACHE_SIZE)
+def get_mid2templateords(db: sqlite3.Connection) -> Dict[int, List[int]]:
+    """ Get mapping of model ID to available templates ids
+
+    Args:
+        db:
+
+    Returns:
+
+    """
+    if get_db_version(db) == 0:
+        minfo = get_model_info(db)
+        return {mid: [x["ord"] for x in minfo[mid]["tmpls"]] for mid in minfo}
+    elif get_db_version(db) == 1:
+        tinfo = read_info(db, "templates")
+        return {int(mid): [int(x) for x in tinfo[mid]] for mid in tinfo}
+    else:
+        raise NotImplementedError
 
 
 @lru_cache(CACHE_SIZE)
