@@ -4,12 +4,18 @@
 import unittest
 
 # ours
-from ankipandas.util.types import *
+from ankipandas.util.types import (
+    is_list_like,
+    is_list_dict_like,
+    is_dict_list_like,
+    is_list_list_like,
+)
+from ankipandas.util.log import set_debug_log_level
 
 
 class TestTypes(unittest.TestCase):
     def setUp(self):
-        pass
+        set_debug_log_level()
 
     def test_is_list_like(self):
         self.assertTrue(is_list_like([1, 2]))
